@@ -804,6 +804,9 @@ class Defaults(object):
 		r'*.h', r'*.hh', r'*.hxx', r'*.hpp', r'*.h++', r'*.ixx', r'*.inc', r'*.markdown', r'*.md', r'*.dox'
 	}
 	# code block syntax highlighting only
+	#
+	# note: we don't need to be comprehensive for the std namespace symbols;
+	# those will get added from the cppreference tagfile
 	cb_enums = {r'(?:std::)?ios(?:_base)?::(?:app|binary|in|out|trunc|ate)'}
 	cb_macros = {
 		# standard builtins:
@@ -918,12 +921,10 @@ class Defaults(object):
 		r'[a-zA-Z_][a-zA-Z_0-9]*_t(?:ype(?:def)?|raits)?',
 		regex_trie(r'bool', r'char', r'double', r'float', r'int', r'long', r'short', r'signed', r'unsigned'),
 		#------ documentation-only types
-		r'[S-Z][0-9]?',
-		r'Foo',
-		r'Bar',
-		r'[Vv]ec(?:tor)?[1-4][hifd]?',
-		r'[Mm]at(?:rix)?[1-4](?:[xX][1-4])?[hifd]?'
+		r'(?:[a-zA-Z][a-zA-Z_]+::)*?(?:[a-zA-Z][a-zA-Z_]+_type|Foo|Bar|[Vv]ec(?:tor)?[1-4][hifd]?|[Mm]at(?:rix)?[1-4](?:[xX][1-4])?[hifd]?)',
+		r'[S-Z][0-9]?'
 	}
+	cb_functions = {regex_trie(r'std::as_const', r'std::move', r'std::forward')}
 
 
 
@@ -1016,6 +1017,7 @@ class CodeBlocks(object):
 		Optional(r'numeric_literals'): ValueOrArray(str, name=r'numeric_literals'),  # deprecated
 		Optional(r'enums'): ValueOrArray(str, name=r'enums'),
 		Optional(r'namespaces'): ValueOrArray(str, name=r'namespaces'),
+		Optional(r'functions'): ValueOrArray(str, name=r'functions'),
 	}
 
 	def __init__(self, config, macros):
@@ -1023,6 +1025,7 @@ class CodeBlocks(object):
 		self.macros = copy.deepcopy(Defaults.cb_macros)
 		self.enums = copy.deepcopy(Defaults.cb_enums)
 		self.namespaces = copy.deepcopy(Defaults.cb_namespaces)
+		self.functions = copy.deepcopy(Defaults.cb_functions)
 
 		if r'code_blocks' in config:
 			config = config['code_blocks']
@@ -1051,6 +1054,12 @@ class CodeBlocks(object):
 					if namespace:
 						self.namespaces.add(namespace)
 
+			if 'functions' in config:
+				for f in coerce_collection(config['functions']):
+					function = f.strip()
+					if function:
+						self.functions.add(function)
+
 		for k, v in macros.items():
 			define = k
 			bracket = define.find('(')
@@ -1358,6 +1367,7 @@ class Context(object):
 		treat_warnings_as_errors: bool,
 		theme: str,
 		copy_assets: bool,
+		temp_dir: Path = None,
 		**kwargs
 	):
 
@@ -1449,13 +1459,16 @@ class Context(object):
 			assert self.input_dir.is_absolute()
 
 			# root temp dir for this run
-			self.temp_dir = re.sub(r'''[!@#$%^&*()+={}<>;:'"_\\/\n\t -]+''', r'_', str(self.input_dir).strip(r'\/'))
-			if len(self.temp_dir) > 256:
-				self.temp_dir = str(self.input_dir)
-				if not self.case_sensitive_paths:
-					self.temp_dir = self.temp_dir.upper()
-				self.temp_dir = sha1(self.temp_dir)
-			self.temp_dir = Path(dirs.TEMP, self.temp_dir)
+			if temp_dir is not None:
+				self.temp_dir = Path(temp_dir).absolute()
+			else:
+				self.temp_dir = re.sub(r'''[!@#$%^&*()+={}<>;:'"_\\/\n\t -]+''', r'_', str(self.input_dir).strip(r'\/'))
+				if len(self.temp_dir) > 256:
+					self.temp_dir = str(self.input_dir)
+					if not self.case_sensitive_paths:
+						self.temp_dir = self.temp_dir.upper()
+					self.temp_dir = sha1(self.temp_dir)
+				self.temp_dir = Path(dirs.TEMP, self.temp_dir)
 			self.verbose_value(r'Context.temp_dir', self.temp_dir)
 			assert self.temp_dir.is_absolute()
 
@@ -1474,7 +1487,7 @@ class Context(object):
 			self.verbose_value(r'Context.xml_dir', self.xml_dir)
 			assert self.xml_dir.is_absolute()
 
-			# html output path (--xml)
+			# html output path (--html)
 			self.html_dir = Path(self.output_dir, r'html')
 			self.verbose_value(r'Context.html_dir', self.html_dir)
 			assert self.html_dir.is_absolute()
