@@ -48,7 +48,7 @@ def log(logger, msg, level=logging.INFO):
 		return
 	if isinstance(logger, bool):
 		if logger:
-			print(msg, file=sys.stderr if level >= logging.WARNING else sys.stdout)
+			print(msg, file=sys.stderr if level >= logging.WARNING else sys.stdout, flush=True)
 	elif isinstance(logger, logging.Logger):
 		logger.log(level, msg)
 	elif isinstance(logger, io.IOBase):
@@ -85,6 +85,11 @@ def lib_version() -> typing.Tuple[int, int, int]:
 
 
 
+def lib_version_string() -> str:
+	return r'.'.join([str(v) for v in lib_version()])
+
+
+
 def filter_filenames(files, include, exclude):
 	if include is not None:
 		files = [f for f in files if include.search(f.name)]
